@@ -2,59 +2,126 @@ package JavaScript::Writer;
 
 use warnings;
 use strict;
-use v5.8.0;
+use 5.008;
 use overload
     '<<' => \&append,
     '""' => \&as_string;
 
+use self;
+
 use JSON::Syck;
 
-our $VERSION = '0.0.8';
+our $VERSION = '0.1.0';
+
+use Sub::Exporter -setup => {
+    exports => ['js'],
+    groups  => {
+        default =>  [ -all ],
+    }
+};
+
+my $base;
+
+sub js {
+    # Let bareword 'js' also refer to $_[0] when it's used in the callbacks.
+    my $level = 1;
+    my @c = ();
+    my $js;
+    while ( $level < 3 && (!defined($c[3]) || $c[3] eq '(eval)') ) {
+        @c = do {
+            package DB;
+            @DB::args = ();
+            caller($level);
+        };
+        $level++;
+        if (ref($DB::args[0]) eq 'JavaScript::Writer') {
+            $js = $DB::args[0] ;
+            last;
+        }
+    }
+
+    my ($target) = @_;
+    if (defined $js) {
+        $js->{target} = $target if defined $target;
+        return $js;
+    }
+
+    $base = JavaScript::Writer->new() unless defined $base;
+    if (defined $target) {
+        $base->{target} = $target;
+    }
+    
+    return $base;
+
+}
 
 sub new {
-    my $class = shift;
-    if (ref $class) {
+    if (ref(self)) {
+        if (defined $base) {
+            self->{statements} = [];
+            return self;
+        }
         return __PACKAGE__->new;
     }
-    my $self = bless {}, $class;
+    my $self = bless { args }, self;
     $self->{statements} = [];
     return $self;
 }
 
 sub call {
-    my ($self, $function, @args) = @_;
-    push @{$self->{statements}},{
-        object => $self->{object} || undef,
+    my ($function, @args) = args;
+    push @{self->{statements}},{
+        object => self->{object} || undef,
         call => $function,
         args => \@args,
         end_of_call_chain => (!defined wantarray)
     };
-    delete $self->{object};
-    return $self;
+    delete self->{object};
+    return self;
 }
 
 sub append {
-    my ($self, $code, @xs) = @_;
-    push @{$self->{statements}}, { code => $code, @xs };
-    return $self;
+    my ($code, @xs) = args;
+    push @{self->{statements}}, { code => $code, @xs };
+    return self;
 }
 
 sub end {
-    my $self = shift;
-    my $last = $self->{statements}[-1];
+    my $last = self->{statements}[-1];
     $last->{end_of_call_chain} = 1;
-    return $self;
+    return self;
 }
 
 sub object {
-    my ($self, $object) = @_;
-    $self->{object} = $object;
-    return $self;
+    self->{object} = args[0];
+    return self;
 }
 
+sub latter {
+    my ($cb) = args;
+
+    my $timeout = self->{target};
+    $timeout =~ s/ms$//;
+    $timeout =~ s/s$/000/;
+
+    my $jsf = JavaScript::Writer::Function->new;
+    $jsf->body($cb);
+
+    self->append("setTimeout($jsf, $timeout)");
+    return self;
+}
 
 use JavaScript::Writer::Var;
 
+sub let {
+    my %vars = args;
+    my $code = "";
+    while (my ($var, $value) = each %vars) {
+        self->var($var, $value);
+    }
+    return self;
+}
+
 sub var {
     my ($self, $var, $value) = @_;
     my $s = "";
@@ -134,10 +201,9 @@ sub function {
 }
 
 sub as_string {
-    my ($self) = @_;
     my $ret = "";
 
-    for (@{$self->{statements}}) {
+    for (@{self->{statements}}) {
         if (my $f = $_->{call}) {
             my $delimiter = $_->{delimiter} ||
                 ($_->{end_of_call_chain} ? ";" : ".");
@@ -147,7 +213,7 @@ sub as_string {
                     join(",",
                          map {
                              if (ref($_) eq 'CODE') {
-                                 $self->function($_)
+                                 self->function($_)
                              }
                              else {
                                  JSON::Syck::Dump $_
@@ -166,8 +232,7 @@ sub as_string {
 }
 
 sub as_html {
-    my $self = shift;
-    qq{<script type="text/javascript">$self</script>}
+    qq{<script type="text/javascript">${\self->as_string}</script>}
 }
 
 our $AUTOLOAD;
@@ -179,13 +244,8 @@ sub AUTOLOAD {
     return $self->call($function, @_);
 }
 
-
-sub FETCH {
-    my ($self, $obj) = @_;
-    $self->{object} = $obj;
-}
-
 1; # Magic true value required at end of module
+
 __END__
 
 =head1 NAME
@@ -196,15 +256,14 @@ JavaScript::Writer - JavaScript code generation from Perl.
 
     use JavaScript::Writer;
 
-    my $js = JavaScript::Writer->new;
+    # Call alert("Nihao").
+    js->call("alert", "Nihao");
 
-    # Call alert("Nihao")
-    $js->call("alert", "Nihao");
+    # Similar, but display Perl-localized message of "Nihao".
+    js->call("alert", _("Nihao") );
 
-    # Similar, but display Perl-localized message of "Nihao". (that might be "哈囉")
-    $js->call("alert", _("Nihao") );
-
-    # Overload
+    # Output
+    js->as_string
 
 =head1 DESCRIPTION
 
@@ -219,14 +278,38 @@ use its C<call> method to call a certain functions from your library.
 
 =over
 
+=item js( [ $target ] )
+
+This function is exported by default to your namespace. Is the spiffy
+ultimate entry point for generating all kinds of javascripts.
+
+C<js> represents a singleton object of all namespace. Unless used in a
+subroutine passed to construct a JavaScript function, it always refers
+to the same C<JavaScript::Writer> object.
+
+It optionally takes a C<$target> parameter that represents something
+on which you can perform some action. For example, here's the sweet
+way to do C<setTimeout>:
+
+    js("3s")->latter(sub{
+        js->say('something');
+    });
+
 =item new()
 
 Object constructor. Takes nothing, gives you an javascript writer
 object. It can be called as a class method, or an object method.
-However, calling it on objects does not imply cloning the original
-object, but just a shorthand because typing package name is always
+Calling it on objects does not imply cloning the original object, but
+just a shorthand to construct a new one. Typing package name is always
 longer.
 
+One special usage is to say:
+
+    js->new;
+
+This just flush the stored statements in the C<js> object, without
+creating a new object.
+
 =item call( $function, $arg1, $arg2, $arg3, ...)
 
 Call an javascript function with arguments. Arguments are given in
@@ -234,7 +317,6 @@ perl's native form, you don't need to use L<JSON> module to serialized
 it first.  (Unless, of course, that's your purpose: to get a JSON
 string in JavaScript.)
 
-
 =item var( $name, [ $value ] )
 
 Declare a value named $name with a optional default value $value.
@@ -268,6 +350,28 @@ Or something like this;
     print $js->as_string;
     # var a;a = myAjaxGet("/my/foo.json");
 
+=item let( var1 => value1, var2 => value2, ... )
+
+This let you assign multiple variables at once.
+
+=item latter($timeout, sub { ... } )
+
+This is another way saying setTimeout(sub { ... }, $timeout). With
+C<js()> funciton, you can now say:
+
+    js("3s")->latter(sub { ... });
+
+And that gets turned into
+
+    setTimeout(function(){...}, 3000);
+
+You can use "ms" and "s" as the time unit, they means millisecond and
+seconds respectively. Number with any units means milliseconds by
+default.
+
+More complex time representation like C<"3h5m2s">, are not implement
+yet.
+
 =item end()
 
 Assert an end of call chain. Calling this is required if you're
@@ -288,7 +392,6 @@ Which will then generated this javascript code snippet:
 
     Widget.Lightbox.show("Nihao")
 
-
 =item while( $condition => $code_ref )
 
 C<$condition> is a string (yes, just a string for now) of javascript
@@ -382,7 +485,7 @@ Kang-min Liu  C<< <gugod@gugod.org> >>
 
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 2007, Kang-min Liu C<< <gugod@gugod.org> >>.
+Copyright (c) 2007,2008, Kang-min Liu C<< <gugod@gugod.org> >>.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself. See L<perlartistic>.
